@@ -1,4 +1,4 @@
-# Copyright 2016-2021 The Van Valen Lab at the California Institute of
+# Copyright 2016-2022 The Van Valen Lab at the California Institute of
 # Technology (Caltech), with support from the Paul Allen Family Foundation,
 # Google, & National Institutes of Health (NIH) under Grant U24CA224309-01.
 # All rights reserved.
@@ -42,7 +42,8 @@ from deepcell.applications import Application
 
 
 MODEL_PATH = ('https://deepcell-data.s3-us-west-1.amazonaws.com/'
-              'saved-models/MultiplexSegmentation-7.tar.gz')
+              'saved-models/MultiplexSegmentation-9.tar.gz')
+MODEL_HASH = 'a1dfbce2594f927b9112f23a0a1739e0'
 
 
 # pre- and post-processing functions
@@ -83,7 +84,7 @@ def format_output_mesmer(output_list):
         dict: Dict of predictions for whole cell and nuclear.
 
     Raises:
-        ValueError: if model output list is not len(8)
+        ValueError: if model output list is not len(4)
     """
     expected_length = 4
     if len(output_list) != expected_length:
@@ -214,7 +215,7 @@ class Mesmer(Application):
         if model is None:
             archive_path = tf.keras.utils.get_file(
                 'MultiplexSegmentation.tgz', MODEL_PATH,
-                file_hash='e7360e8e87c3ab71ded00a577a61c689',
+                file_hash=MODEL_HASH,
                 extract=True, cache_subdir='models'
             )
             model_path = os.path.splitext(archive_path)[0]
@@ -269,9 +270,9 @@ class Mesmer(Application):
             numpy.array: Instance segmentation mask.
         """
         default_kwargs_cell = {
-            'maxima_threshold': 0.1,
+            'maxima_threshold': 0.075,
             'maxima_smooth': 0,
-            'interior_threshold': 0.3,
+            'interior_threshold': 0.2,
             'interior_smooth': 2,
             'small_objects_threshold': 15,
             'fill_holes_threshold': 15,
@@ -281,7 +282,7 @@ class Mesmer(Application):
         default_kwargs_nuc = {
             'maxima_threshold': 0.1,
             'maxima_smooth': 0,
-            'interior_threshold': 0.3,
+            'interior_threshold': 0.2,
             'interior_smooth': 2,
             'small_objects_threshold': 15,
             'fill_holes_threshold': 15,
